#!/usr/bin/env python

import os.path
import subprocess

DCE_BUILD_DIR = 'dce_build'
DCE_TOP_DIR = os.getcwd()
try:
    os.mkdir(os.path.join(os.getcwd(), DCE_BUILD_DIR))
except OSError:
    print "could't mkdir or already exists"
    exit
try:
    os.mkdir(os.path.join(os.getcwd(), "build"))
    os.mkdir(os.path.join(os.getcwd(), "build/debug"))
except OSError:
    print "could't mkdir or already exists"
    exit

def build_iproute():
    os.chdir(DCE_BUILD_DIR)

    cmd = ["wget", "http://www.linuxgrill.com/anonymous/iproute2/NEW-OSDL/iproute2-2.6.38.tar.bz2"]
    print " => ", ' '.join(cmd)
    subprocess.Popen(cmd).wait()
    cmd = ["tar", "jxf", "iproute2-2.6.38.tar.bz2"]
    print " => ", ' '.join(cmd)
    subprocess.Popen(cmd).wait()

    os.chdir("./iproute2-2.6.38/")

    cmd = ["./configure"]
    print " => ", ' '.join(cmd)
    subprocess.Popen(cmd).wait()

    cmd = ["make", "CCOPTS='-fPIC'", "LDFLAGS=-pie"]
    print " => ", ' '.join(cmd)
    subprocess.Popen(cmd).wait()

    cmd = ["cp", "ip/ip", DCE_TOP_DIR + "/build/debug"]
    print " => ", ' '.join(cmd)
    subprocess.Popen(cmd).wait()
    os.chdir(DCE_TOP_DIR)

def build_umip():
    os.chdir(DCE_BUILD_DIR)
    UMIP_GIT_URL = 'git://git.umip.org/umip.git'
    UMIP_GIT_DIR = 'umip'

    """
    Fetches and build UMIP from public repository.
    Returns True if successful, False it not.
    """

    try:
        os.mkdir(os.path.join(os.getcwd(), "umip"))
    except OSError:
        pass

    print "Trying to fetch umip; this will fail if no network connection is available.  Hit Ctrl-C to skip."
    os.chdir("./umip/")
    cmd = ["git", "clone", UMIP_GIT_URL]
    print " => ", ' '.join(cmd)
    try:
        if subprocess.Popen(cmd).wait():
            print "go ahead"
    except KeyboardInterrupt:
        print "Interrupted; Canceld UMIP fetch."
        exit

    # Configure umip
    os.chdir("./umip/")
    cmd = ["autoreconf", "-i"]
    print " => ", ' '.join(cmd)
    subprocess.Popen(cmd).wait()
    env = {"CFLAGS": "-fPIC -g", "CXXFLAGS": "-fPIC -g", "LDFLAGS": "-pie -g", "PATH": "/bin/:/usr/bin/"}
    cmd = ["./configure", "--enable-vt", "--with-builtin-crypto"]
    print " => ", ' '.join(cmd)
    subprocess.Popen(cmd,env=env).wait()

    # build umip
    cmd = ["make"]
    print " => ", ' '.join(cmd)
    subprocess.Popen(cmd).wait()

    # install umip
    cmd = ["cp", "src/mip6d", DCE_TOP_DIR + "/build/debug"]
    print " => ", ' '.join(cmd)
    subprocess.Popen(cmd).wait()
    os.chdir(DCE_TOP_DIR)

def build_quagga():
    os.chdir(DCE_BUILD_DIR)
    QUAGGA_HG_URL = 'http://www.sfc.wide.ad.jp/~tazaki/hg/zebra-manemo'
    QUAGGA_HG_DIR = 'zebra-manemo'
    """
    Fetches and build quagga from private repository.
    Returns True if successful, False it not.
    """
        
    print "Trying to fetch quagga; this will fail if no network connection is available.  Hit Ctrl-C to skip."
    cmd = ["hg", "clone", QUAGGA_HG_URL]
    print " => ", ' '.join(cmd)
    try:
        if subprocess.Popen(cmd).wait():
            print "go ahead"
    except KeyboardInterrupt:
        print "Interrupted; Canceld Quagga fetch."
        return False

    # Configure quagga
    os.chdir("./zebra-manemo/zebra/")
    cmd = ["./autogen.sh"]
    print " => ", ' '.join(cmd)
    subprocess.Popen(cmd).wait()
    cmd = ["./configure", "--with-cflags=-g -pie -fPIE", "--enable-tcp-zebra"]
    print " => ", ' '.join(cmd)
    subprocess.Popen(cmd).wait()

    # build quagga
    cmd = ["make"]
    print " => ", ' '.join(cmd)
    subprocess.Popen(cmd).wait()

    # install quagga
    cmd = ["cp", "zebra/zebra", DCE_TOP_DIR + "/build/debug"]
    print " => ", ' '.join(cmd)
    subprocess.Popen(cmd).wait()
    cmd = ["cp", "ospfd/ospfd", DCE_TOP_DIR + "/build/debug"]
    print " => ", ' '.join(cmd)
    subprocess.Popen(cmd).wait()
    cmd = ["cp", "bgpd/bgpd", DCE_TOP_DIR + "/build/debug"]
    print " => ", ' '.join(cmd)
    subprocess.Popen(cmd).wait()
    cmd = ["cp", "olsr6d/olsr6d", DCE_TOP_DIR + "/build/debug"]
    print " => ", ' '.join(cmd)
    subprocess.Popen(cmd).wait()

    os.chdir(DCE_TOP_DIR)

def build_ns_3_linux():
    os.chdir(DCE_BUILD_DIR)
    NS3LINUX_HG_URL = 'http://code.nsnam.org/mathieu/ns-3-linux'
    NS3LINUX_HG_REV = '971f696ab6b2'

    cmd = ["hg", "clone", "-r", NS3LINUX_HG_REV, NS3LINUX_HG_URL]
    print " => ", ' '.join(cmd)
    try:
        if subprocess.Popen(cmd).wait():
            print "go ahead"
    except KeyboardInterrupt:
        print "Interrupted; Canceld ns-3-linux fetch."
        return False

    os.chdir("./ns-3-linux/")
    cmd = ["git", "clone", "git://git.kernel.org/pub/scm/linux/kernel/git/davem/net-next.git", "net-next-2.6"]
    print " => ", ' '.join(cmd)
    subprocess.Popen(cmd).wait()

    # it will failed under ubuntu (cause /bin/dash)
    os.chdir("./net-next-2.6/")
    cmd = ["git", "reset", "--hard", "fed66381d65a35198639f564365e61a7f256bf79"]
    print " => ", ' '.join(cmd)
    subprocess.Popen(cmd).wait()
    os.chdir("../")
    cmd = ["touch", ".download.ts"]
    print " => ", ' '.join(cmd)
    subprocess.Popen(cmd).wait()

    cmd = ["/bin/bash", "-c", "make .patch.ts"]
    print " => ", ' '.join(cmd)
    subprocess.Popen(cmd).wait()

    cmd = ["ln", "-s", DCE_TOP_DIR + "/" + DCE_BUILD_DIR + "/ns-3-linux/sim", "net-next-2.6/arch/sim"]
    print " => ", ' '.join(cmd)
    subprocess.Popen(cmd).wait()

    cmd = ["make", "defconfig"]
    print " => ", ' '.join(cmd)
    subprocess.Popen(cmd).wait()

    # apply poll patch 
    PATCH_REV = '794b7dafafa8'
    cmd = ["wget", "http://www.sfc.wide.ad.jp/~tazaki/hg/ns-3-linux-patches/raw-file/" + PATCH_REV + "/poll_read.patch"]
    print " => ", ' '.join(cmd)
    subprocess.Popen(cmd).wait()
    cmd = ["wget", "http://www.sfc.wide.ad.jp/~tazaki/hg/ns-3-linux-patches/raw-file/" + PATCH_REV + "/ubuntu10_amd64.patch"]
    print " => ", ' '.join(cmd)
    subprocess.Popen(cmd).wait()
    patch = ["cat", "poll_read.patch"]
    cmd = ["patch", "-p1"]
    print " => ", ' '.join(cmd), '| '.join(patch)
    p1 = subprocess.Popen(patch, stdout=subprocess.PIPE)
    subprocess.Popen(cmd, stdin=p1.stdout).wait()
    cmd = ["cp", "quagga-umip.config", "net-next-2.6/.config"]
    print " => ", ' '.join(cmd)
    subprocess.Popen(cmd).wait()
    patch = ["cat", "ubuntu10_amd64.patch"]
    cmd = ["patch", "-p1"]
    print " => ", ' '.join(cmd), '| '.join(patch)
    p1 = subprocess.Popen(patch, stdout=subprocess.PIPE)
    subprocess.Popen(cmd, stdin=p1.stdout).wait()

    cmd = ["make"]
    print " => ", ' '.join(cmd)
    subprocess.Popen(cmd).wait()

    os.chdir(DCE_TOP_DIR)

def build_pybindgen():
    os.chdir(DCE_BUILD_DIR)
    PYBINDGEN_BZR_URL = 'https://launchpad.net/pybindgen'
    PYBINDGEN_BZR_REV = '770'
    PYBINDGEN_DEST_DIR = 'pybindgen-0.15.0.770'

    #bzr checkout -rrevno:770 https://launchpad.net/pybindgen pybindgen-0.15.0.770
    cmd = ["bzr", "checkout", "-rrevno:" + PYBINDGEN_BZR_REV, PYBINDGEN_BZR_URL, PYBINDGEN_DEST_DIR]
    print " => ", ' '.join(cmd)
    try:
        if subprocess.Popen(cmd).wait():
            print "go go"
    except KeyboardInterrupt:
        print "Interrupted; Canceld pybindgen fetch."
        return False

    os.chdir(PYBINDGEN_DEST_DIR)

    cmd = ["./waf", "configure"]
    print " => ", ' '.join(cmd)
    subprocess.Popen(cmd).wait()
    cmd = ["./waf"]
    print " => ", ' '.join(cmd)
    subprocess.Popen(cmd).wait()
    os.chdir(DCE_TOP_DIR)




build_iproute ()
build_umip ()
build_quagga ()
build_ns_3_linux()
build_pybindgen()
