#include "ns3/helper-module.h"
#include "ns3/process-module.h"
#include "ns3/simulator-module.h"
#include "ns3/core-module.h"
#include "ns3/node-module.h"

using namespace ns3;

int main (int argc, char *argv[])
{
  CommandLine cmd;
  cmd.Parse (argc, argv);

  Config::SetGlobal ("ChecksumEnabled", BooleanValue (true));

  NodeContainer nodes;
  nodes.Create (3);
  NodeContainer a = NodeContainer (nodes.Get (0), nodes.Get (1));
  NodeContainer b = NodeContainer (nodes.Get (1), nodes.Get (2));

  PointToPointHelper p2p;
  p2p.SetDeviceAttribute ("DataRate", StringValue ("5Mbps"));
  p2p.SetChannelAttribute ("Delay", StringValue ("2ms"));
  NetDeviceContainer devA = p2p.Install (a);
  NetDeviceContainer devB = p2p.Install (b);

  // add an ip stack
  InternetStackHelper stack;
  stack.Install (nodes);

  // ip address assignment
  Ipv4AddressHelper ipv4;
  ipv4.SetBase ("10.1.1.0", "255.255.255.0");
  ipv4.Assign (devA);
  ipv4.SetBase ("10.1.2.0", "255.255.255.0");
  ipv4.Assign (devB);

  // setup ip routes
  Ipv4GlobalRoutingHelper::PopulateRoutingTables ();

  // start some stupid traffic
  InetSocketAddress udpDst (Ipv4Address ("10.1.2.2"), 10);
  OnOffHelper onoff ("ns3::UdpSocketFactory", udpDst);
  onoff.SetAttribute ("OnTime", RandomVariableValue (ConstantVariable (1)));
  onoff.SetAttribute ("OffTime", RandomVariableValue (ConstantVariable (0)));
  onoff.SetAttribute ("PacketSize", StringValue ("1110"));
  onoff.SetAttribute ("DataRate", StringValue ("4Mb/s"));
  ApplicationContainer app = onoff.Install (nodes.Get (1));
  app.Start (Seconds (1.0));
  app.Stop (Seconds (10.0));

  PacketSinkHelper sink ("ns3::UdpSocketFactory", udpDst);
  app = sink.Install (nodes.Get (2));
  app.Start (Seconds (1.0));
  app.Stop (Seconds (10.0));


  // start the ping binaries.
  ProcessManagerHelper processManager;
  processManager.Install (nodes);
  ProcessHelper process;
  process.SetBinary ("build/debug/ping");
  process.SetStackSize (1<<16);
  process.AddArgument ("10.1.2.2");
  process.AddArgument ("-c");
  process.AddArgument ("10");
  ApplicationContainer apps = process.Install (nodes.Get (0));
  apps.Start (Seconds (1.0));
  apps = process.Install (nodes.Get (1));
  apps.Start (Seconds (1.0));

  PointToPointHelper::EnablePcapAll ("process-ping");

  Simulator::Run ();
  Simulator::Destroy ();


  return 0;
}
