#include "ns3-socket-fd-factory.h"
#include "unix-fd.h"
#include "unix-socket-fd.h"
#include "unix-datagram-socket-fd.h"
#include "unix-stream-socket-fd.h"
#include "ns3/log.h"
#include "utils.h"
#include "process.h"
#include "netlink/netlink-socket-factory.h"
#include "ns3/socket-factory.h"
#include "ns3/socket.h"
#include "ns3/uinteger.h"
#include "ns3/node.h"
#include <errno.h>
#include <linux/netlink.h>
#include <linux/rtnetlink.h>

NS_LOG_COMPONENT_DEFINE("Ns3SocketFdFactory");

namespace ns3 {

NS_OBJECT_ENSURE_REGISTERED(Ns3SocketFdFactory);

TypeId 
Ns3SocketFdFactory::GetTypeId (void)
{
  static TypeId tid = TypeId ("ns3::Ns3SocketFdFactory")
    .SetParent<SocketFdFactory> ()
    .AddConstructor<Ns3SocketFdFactory> ()
    ;
  return tid;
}

Ns3SocketFdFactory::Ns3SocketFdFactory ()
  : m_netlink (0)
{
}
void 
Ns3SocketFdFactory::NotifyNewAggregate (void)
{
  Ptr<Node> node = this->GetObject<Node> ();
  if (m_netlink == 0)
    {
      m_netlink = CreateObject<NetlinkSocketFactory> ();
      node->AggregateObject (m_netlink);
    }
}
UnixFd *
Ns3SocketFdFactory::CreateSocket (int domain, int type, int protocol)
{
  GET_CURRENT(this << domain << type << protocol);
  NS_ASSERT (domain == PF_INET || domain == PF_NETLINK || domain == PF_INET6);

  UnixSocketFd *socket = 0;
  Ptr<Socket> sock;

  if (domain == AF_INET)
    {
      switch (type) 
        {
        case SOCK_DGRAM:
          {
            TypeId tid = TypeId::LookupByName ("ns3::UdpSocketFactory");
            Ptr<SocketFactory> factory = GetObject<SocketFactory> (tid);
            sock = factory->CreateSocket ();
            socket = new UnixDatagramSocketFd (sock);
            break;
          }
        case SOCK_RAW:
          {
            TypeId tid = TypeId::LookupByName ("ns3::Ipv4RawSocketFactory");
            Ptr<SocketFactory> factory = GetObject<SocketFactory> (tid);
            sock = factory->CreateSocket ();
            sock->SetAttribute ("Protocol", UintegerValue (protocol));
            socket = new UnixDatagramSocketFd (sock);
            break;
          }
        case SOCK_STREAM:
          {
            TypeId tid = TypeId::LookupByName ("ns3::TcpSocketFactory");
            Ptr<SocketFactory> factory = GetObject<SocketFactory> (tid);
            sock = factory->CreateSocket ();
            socket = new UnixStreamSocketFd (sock);
            break;
          }
        }
    }
  else if (domain == AF_NETLINK)
    {
      if (type != SOCK_DGRAM && type != SOCK_RAW)
        {
          current->err = ESOCKTNOSUPPORT;
          return 0;
        }
      /*for now we only support NETLINK_ROUTE*/
      if (protocol != NETLINK_ROUTE)
        {
          current->err = EPROTONOSUPPORT;
          return 0;
        }
      
      TypeId tid= TypeId::LookupByName ("ns3::NetlinkSocketFactory");
      Ptr<SocketFactory> factory = GetObject<SocketFactory> (tid);
      sock = factory->CreateSocket ();
      socket = new UnixDatagramSocketFd (sock);
    }
  else
    {
      //      NS_FATAL_ERROR ("missing socket type");
      return 0;
    }

  return socket;
}

} // namespace ns3
